@@ -16,6 +16,7 @@ from datetime import timedelta
 
 from binance.client import Client
 
+import math
 
 class NaPoleonBinanceSpotBot(object):
 
@@ -79,19 +80,33 @@ class NaPoleonBinanceSpotBot(object):
         columns_names = ['time', 'open', 'high', 'low', 'close', 'volume', 'close_time', 'quote_asset_vol',
                              'ntrades',
                              'takerbuy_base_asset_vol', 'takerbuy_quote_asset_vol', 'ignore']
-
         dataframe.columns = columns_names
-
         dataframe['time'] = pd.to_datetime(dataframe['time'], unit='ms')
         dataframe = dataframe.sort_values(by=['time'])
         dataframe = dataframe.rename(columns={"time": "date"}, errors="raise")
         dataframe = dataframe.set_index(dataframe['date'])
         dataframe = dataframe[['open', 'high', 'low', 'close', 'volume']].copy()
-
         print('size fetched')
         print(dataframe.shape)
         return dataframe
 
+
+    def get_hourly_live_ohlc(self, pair):
+        rbtc = requests.get('https://min-api.cryptocompare.com/data/v2/histoday?fsym={}&tsym=USD&limit=1500'.format(pair.replace('USDT','')))
+        df = pd.DataFrame(json.loads(rbtc.text)['Data']['Data'])
+        df = df.set_index('time')
+        df.drop(columns=['high', 'low', 'volumefrom', 'volumeto', 'open', 'conversionType', 'conversionSymbol'],
+                inplace=True)
+        df.rename(columns={'close': pair}, inplace=True)
+        print('data loaded')
+
+        df = df.reset_index()
+        df['Date'] = df.apply(lambda x: datetime.datetime.fromtimestamp(x['time'], tz=datetime.timezone.utc), axis=1)
+        df['weekday'] = df.apply(lambda x: datetime.datetime.date(x['Date']).weekday(), axis=1)
+        df = df.set_index('Date')
+        df.drop(columns=['time'], inplace=True)
+        return df
+
     def get_hourly_ohlc(self, pair, starting_date, ending_date):
         #starting_date_string = starting_date.strftime('%m/%d/%y')
         #ending_date_string = ending_date.strftime('%m/%d/%y')
@@ -145,28 +160,263 @@ class NaPoleonBinanceSpotBot(object):
         spot_universe = [asset['asset'] for asset in account['balances']]
         return spot_universe
 
+    def get_spot_balance_df(self,):
+        account = self.client.get_account()
+        balances_df = pd.DataFrame(account['balances'])
+        return balances_df
+
     def get_spot_balance(self,):
         account = self.client.get_account()
         return account['balances']
 
+    def get_last_ticker_underlyings(self, ticker):
+        ticker_info = self.client.get_ticker(symbol = ticker)
+        return ticker_info
 
-    def cancel_all_order(self, pair):
-        Cleaner = self.exchange.fetch_open_orders(pair)
-        L = len(Cleaner)
-        if L > 0:
-            for i in range(0, L):
-                self.exchange.cancel_order(Cleaner[i]['info']['orderId'], 'BTC/USDT')
+    def get_total_valo(self):
+        spot_balances = self.get_free_spot_balance()
+        total_valo = 0.
+        for me_ticker, me_balance in spot_balances.items():
+            if me_ticker != 'USDT':
+                me_pair = me_ticker + 'USDT'
+                print(f'pair {me_pair}')
+                ticker_info = self.get_last_ticker_underlyings(me_pair)
+                price = float(ticker_info['bidPrice'])
+                print(f'ticker {me_ticker} price {price}')
+                total_valo = total_valo + me_balance * price
+            else:
+                total_valo = total_valo + me_balance
+        return total_valo
 
-    def cancel_all_orders(self):
-        self.cancel_all_order('BTC/USDT')
-        self.cancel_all_order('ETH/USDT')
-        self.cancel_all_order('LTC/USDT')
-        self.cancel_all_order('EOS/USDT')
-        self.cancel_all_order('XRP/USDT')
-        self.cancel_all_order('BCH/USDT')
-        self.cancel_all_order('TRX/USDT')
+    def enter_coins(self, quantities_to_enter=None):
+        for me_ticker, me_quantity in quantities_to_enter.items():
+            me_pair=me_ticker + 'USDT'
+            info = self.client.get_symbol_info(me_pair)
+            minQty = info['filters'][2]['minQty']
+            to_roundto = -math.log10(float(minQty))
+#            factor = 1/float(minQty)
+            factor = math.pow(10,to_roundto)
+            me_quantity= math.floor(me_quantity * factor) / factor
+            if me_quantity > float(minQty):
+                not_success = True
+                counter = 0
+                while not_success and counter<=5:
+                    try:
+                        print(f'buying {me_ticker} for {me_quantity}')
+                        order = self.client.order_market_buy(symbol=me_pair, quantity=me_quantity)
+                        print(order)
+                        not_success=False
+                    except Exception as inst:
+                        print(inst)
+                        me_quantity = me_quantity - float(minQty)
+                        counter = counter+1
+                if counter > 5 :
+                    raise Exception('holy guacamole')
 
 
+    def liquidate_coins(self, quantities_to_liquidate=None):
+        for me_ticker, me_quantity in quantities_to_liquidate.items():
+            me_pair=me_ticker + 'USDT'
+            info = self.client.get_symbol_info(me_pair)
+            minQty = info['filters'][2]['minQty']
+            to_roundto = -math.log10(float(minQty))
+            #factor = 1/float(minQty)
+            factor = math.pow(10, to_roundto)
+            me_quantity= math.floor(me_quantity * factor) / factor
+            if me_quantity>float(minQty):
+                print(f'selling {me_ticker} for {me_quantity}')
+                try:
+                    order = self.client.order_market_sell(symbol=me_ticker + 'USDT', quantity=me_quantity)
+                    print(order)
+                except Exception as inst:
+                    print(inst)
+
+
+
+    def get_coins_to_enter(self, signals):
+        old_quantities = self.get_free_spot_balance()
+        total_valo = 0.
+        spot_prices = {}
+        old_tickers = list(old_quantities.keys())
+        new_tickers = list(signals.keys())
+        for me_ticker, me_balance in old_quantities.items():
+            #only the USDT is reinvested
+            if me_ticker == 'USDT':
+                total_valo = total_valo + me_balance
+
+        for me_ticker in signals.keys():
+            me_pair = me_ticker + 'USDT'
+            print(f'pair {me_pair}')
+            ticker_info = self.get_last_ticker_underlyings(me_pair)
+            price = float(ticker_info['bidPrice'])
+            print(f'ticker {me_ticker} price {price}')
+            spot_prices[me_ticker] = price
+        coin_to_enter_number = 0
+        for me_ticker, me_sig in signals.items():
+            if me_ticker not in old_quantities.keys():
+                coin_to_enter_number = coin_to_enter_number+1
+
+        usdt_new_quantities = {}
+        ticker_new_quantities = {}
+        for me_ticker, me_sig in signals.items():
+            if me_ticker not in old_quantities.keys():
+                usdt_new_quantities[me_ticker] = total_valo/coin_to_enter_number
+                ticker_new_quantities[me_ticker] = usdt_new_quantities[me_ticker] /spot_prices[me_ticker]
+
+        return ticker_new_quantities
+
+
+    def get_coins_to_liquidate(self, signals):
+        old_quantities = self.get_free_spot_balance()
+        total_valo = 0.
+        spot_prices = {}
+        old_tickers = list(old_quantities.keys())
+        new_tickers = list(signals.keys())
+        for me_ticker, me_balance in old_quantities.items():
+            if me_ticker != 'USDT':
+                me_pair = me_ticker + 'USDT'
+                print(f'pair {me_pair}')
+                ticker_info = self.get_last_ticker_underlyings(me_pair)
+                price = float(ticker_info['bidPrice'])
+                print(f'ticker {me_ticker} price {price}')
+                spot_prices[me_ticker] = price
+                total_valo = total_valo + me_balance * price
+            else:
+                total_valo = total_valo + me_balance
+
+        for me_ticker in signals.keys():
+            me_pair = me_ticker + 'USDT'
+            print(f'pair {me_pair}')
+            ticker_info = self.get_last_ticker_underlyings(me_pair)
+            price = float(ticker_info['bidPrice'])
+            print(f'ticker {me_ticker} price {price}')
+            spot_prices[me_ticker] = price
+
+        usdt_new_quantities = {}
+        ticker_new_quantities = {}
+        for me_ticker, me_sig in signals.items():
+            usdt_new_quantities[me_ticker] = me_sig*total_valo
+            ticker_new_quantities[me_ticker] = me_sig*total_valo/spot_prices[me_ticker]
+
+        ##### what is in old but not in new
+        liquidations = {}
+        for me_ticker, me_quantity in old_quantities.items():
+            if me_ticker not in ticker_new_quantities.keys():
+                print(f'liquidating quantity {me_ticker} {me_quantity}')
+                if me_ticker != 'USDT':
+                    if abs(me_quantity) > 1e-3:
+                        liquidations[me_ticker]=me_quantity
+            # else :
+            #     quantity_to_pass = ticker_new_quantities[me_ticker] - old_quantities[me_ticker]
+            #     print(f'passing quantity {me_ticker} {quantity_to_pass}')
+        return liquidations
+
+    def get_quantity_to_pass(self,signals=None):
+        old_quantities = self.get_free_spot_balance()
+        total_valo = 0.
+        spot_prices = {}
+
+        old_tickers = list(old_quantities.keys())
+        old_tickers = [old_tick for old_tick in old_tickers if old_tick != 'USDT']
+        new_tickers = list(signals.keys())
+
+        for me_ticker, me_balance in old_quantities.items():
+            if me_ticker != 'USDT':
+                me_pair = me_ticker + 'USDT'
+                print(f'pair {me_pair}')
+                ticker_info = self.get_last_ticker_underlyings(me_pair)
+                price = float(ticker_info['bidPrice'])
+                print(f'ticker {me_ticker} price {price}')
+                spot_prices[me_ticker] = price
+                total_valo = total_valo + me_balance * price
+            else:
+                total_valo = total_valo + me_balance
+
+        for me_ticker in signals.keys():
+            me_pair = me_ticker + 'USDT'
+            print(f'pair {me_pair}')
+            ticker_info = self.get_last_ticker_underlyings(me_pair)
+            price = float(ticker_info['bidPrice'])
+            print(f'ticker {me_ticker} price {price}')
+            spot_prices[me_ticker] = price
+
+        usdt_new_quantities = {}
+        ticker_new_quantities = {}
+        for me_ticker, me_sig in signals.items():
+            usdt_new_quantities[me_ticker] = me_sig*total_valo
+            ticker_new_quantities[me_ticker] = me_sig*total_valo/spot_prices[me_ticker]
+
+        ##### what is in old but not in new
+        liquidations = {}
+        for me_ticker, me_quantity in old_quantities.items():
+            if me_ticker not in ticker_new_quantities.keys():
+                print(f'liquidating quantity {me_ticker} {me_quantity}')
+                liquidations[me_ticker]=me_quantity
+            # else :
+            #     quantity_to_pass = ticker_new_quantities[me_ticker] - old_quantities[me_ticker]
+            #     print(f'passing quantity {me_ticker} {quantity_to_pass}')
+
+        enterings = {}
+        ##### what is in new but not in old
+        for me_ticker, me_quantity in ticker_new_quantities.items():
+            if me_ticker not in old_quantities.keys():
+                print(f'entering quantity {me_ticker} {me_quantity}')
+                enterings[me_ticker] = me_quantity
+
+        switchs = {}
+        for me_ticker, me_quantity in old_quantities.items():
+            if me_ticker in ticker_new_quantities.keys():
+                quantity_to_pass = ticker_new_quantities[me_ticker] - old_quantities[me_ticker]
+                print(f'passing quantity {me_ticker} {quantity_to_pass}')
+                switchs[me_ticker] = me_quantity
+
+        return liquidations,enterings,switchs
+
+
+    def get_free_spot_balance(self,):
+        account = self.client.get_account()
+        balances = account['balances']
+        free_positive_balances={}
+        for asset_balance in balances :
+            if float(asset_balance['free'])>0:
+                me_quantity =float(asset_balance['free'])
+                if abs(me_quantity)>1e-3:
+                    free_positive_balances[asset_balance['asset']] = me_quantity
+        return free_positive_balances
+
+    def get_all_spot_open_orders(self):
+        me_orders = self.client.get_open_orders()
+        #return pd.DataFrame(me_orders)
+        return me_orders
+
+    def cancel_all_spot_open_orders(self):
+        for me_order in self.get_all_spot_open_orders():
+            self.client.cancel_order(symbol = me_order['symbol'], orderId = me_order['orderId'])
+
+    def get_spot_orders(self, pair):
+        me_orders = self.client.get_all_orders(symbol = pair)
+        return me_orders
+
+    def cancel_all_spot_orders(self,):
+        all_orders = self.client.get_all_orders()
+        print(all_orders)
+
+    # def cancel_all_order(self, pair):
+    #     Cleaner = self.exchange.fetch_open_orders(pair)
+    #     L = len(Cleaner)
+    #     if L > 0:
+    #         for i in range(0, L):
+    #             self.exchange.cancel_order(Cleaner[i]['info']['orderId'], 'BTC/USDT')
+    #
+    # def cancel_all_orders(self):
+    #     self.cancel_all_order('BTC/USDT')
+    #     self.cancel_all_order('ETH/USDT')
+    #     self.cancel_all_order('LTC/USDT')
+    #     self.cancel_all_order('EOS/USDT')
+    #     self.cancel_all_order('XRP/USDT')
+    #     self.cancel_all_order('BCH/USDT')
+    #     self.cancel_all_order('TRX/USDT')
+
 
     def get_futures_balance_history(self, start_time, end_time):
         startTime = int(start_time.timestamp() * 1000)
@@ -235,177 +485,8 @@ class NaPoleonBinanceSpotBot(object):
             return positions_margin_change_df[['date', 'asset', 'amount']]
         return positions_margin_change_df
 
-    def get_futures_position_margin_change_history(self, asset, start_time, end_time):
 
-        startTime = int(start_time.timestamp() * 1000)
-        endTime = int(end_time.timestamp() * 1000)
 
-        BASE_URL = 'https://fapi.binance.com'
-        PATH = '/fapi/v1/positionMargin/history'
-
-        headers = {
-            'X-MBX-APIKEY': self.apiKey
-        }
-        timestamp = int(time.time() * 1000)
-
-        params = {
-            'symbol': asset,
-            'type': 1,
-            'recvWindow': 60000,
-            'timestamp': timestamp,
-            'startTime': startTime,
-            'endTime': endTime,
-            ##            'limit': 500,
-            ##           'type':2
-        }
-        query_string = urllib.parse.urlencode(params)
-        params['signature'] = hmac.new(self.secret.encode('utf-8'), query_string.encode('utf-8'),
-                                       hashlib.sha256).hexdigest()
-        url = urljoin(BASE_URL, PATH)
-        print(url)
-        r = requests.get(url, headers=headers, params=params)
-        dataSet = r.json()
-        if dataSet['total'] > 0:
-            transfers = dataSet['rows']
-
-            transfer_df = pd.DataFrame(transfers)
-
-            transfer_df['date'] = pd.to_datetime(transfer_df['timestamp'], unit='ms')
-
-            return transfer_df
-        else:
-            return None
-
-    def get_future_position_risk(self):
-
-        BASE_URL = 'https://fapi.binance.com'
-        PATH = '/fapi/v2/positionRisk'
-
-        headers = {
-            'X-MBX-APIKEY': self.apiKey
-        }
-        timestamp = int(time.time() * 1000)
-
-        params = {
-            'recvWindow': 60000,
-            'timestamp': timestamp,
-        }
-        query_string = urllib.parse.urlencode(params)
-        params['signature'] = hmac.new(self.secret.encode('utf-8'), query_string.encode('utf-8'),
-                                       hashlib.sha256).hexdigest()
-        url = urljoin(BASE_URL, PATH)
-        print(url)
-        r = requests.get(url, headers=headers, params=params)
-        dataSet = r.json()
-        return dataSet
-
-    def get_futures_transfer_history(self, assets, start_time, end_time):
-        transfer_df = None
-        for asset in assets:
-            if transfer_df is None:
-                transfer_df = self.get_coin_futures_transfer_history(asset, start_time, end_time)
-            else:
-                add_transfer_df = self.get_coin_futures_transfer_history(asset, start_time, end_time)
-                if add_transfer_df is not None:
-                    transfer_df = pd.concat([transfer_df, add_transfer_df])
-        if transfer_df is not None:
-            return transfer_df[['date', 'asset', 'amount', 'type']]
-        return transfer_df
-
-    def get_top_trader_long_short_ration(self, asset, start_time, end_time, period):
-
-        startTime = int(start_time.timestamp() * 1000)
-        endTime = int(end_time.timestamp() * 1000)
-
-        BASE_URL = 'https://fapi.binance.com'
-        PATH = '/futures/data/topLongShortPositionRatio'
-
-        headers = {
-            'X-MBX-APIKEY': self.apiKey
-        }
-
-        params = {
-            'symbol': asset,
-            'period': period,
-            'startTime': startTime,
-            'endTime': endTime,
-            'limit': 30
-
-        }
-        query_string = urllib.parse.urlencode(params)
-        params['signature'] = hmac.new(self.secret.encode('utf-8'), query_string.encode('utf-8'),
-                                       hashlib.sha256).hexdigest()
-        url = urljoin(BASE_URL, PATH)
-        print(url)
-        r = requests.get(url, headers=headers, params=params)
-        dataSet = r.json()
-        ls_ratio_df = pd.DataFrame(dataSet)
-        ls_ratio_df.index = pd.to_datetime(ls_ratio_df['timestamp'], unit='ms')
-
-        return ls_ratio_df
-
-    def get_coin_futures_transfer_history(self, asset, start_time, end_time):
-
-        startTime = int(start_time.timestamp() * 1000)
-        endTime = int(end_time.timestamp() * 1000)
-
-        BASE_URL = 'https://api.binance.com'
-        PATH = '/sapi/v1/futures/transfer'
-
-        headers = {
-            'X-MBX-APIKEY': self.apiKey
-        }
-        timestamp = int(time.time() * 1000)
-
-        params = {
-            'asset': asset,
-            'recvWindow': 60000,
-            'timestamp': timestamp,
-            'startTime': startTime,
-            'endTime': endTime,
-            'limit': 30
-
-        }
-        query_string = urllib.parse.urlencode(params)
-        params['signature'] = hmac.new(self.secret.encode('utf-8'), query_string.encode('utf-8'),
-                                       hashlib.sha256).hexdigest()
-        url = urljoin(BASE_URL, PATH)
-        print(url)
-        r = requests.get(url, headers=headers, params=params)
-        dataSet = r.json()
-        if dataSet['total'] > 0:
-            transfers = dataSet['rows']
-            transfer_df = pd.DataFrame(transfers)
-
-            transfer_df['date'] = pd.to_datetime(transfer_df['timestamp'], unit='ms')
-
-            return transfer_df
-        else:
-            return None
-
-    def get_balance_low_level(self, request_date):
-        BASE_URL = 'https://api.binance.com'
-        PATH = '/api/v3/account'
-        if request_date is None:
-            timestamp = int(time.time() * 1000)
-        else:
-            timestamp = calendar.timegm(request_date.timetuple())
-
-        headers = {
-            'X-MBX-APIKEY': self.apiKey
-        }
-        params = {
-            'recvWindow': 5000,
-            'timestamp': timestamp,
-        }
-        query_string = urllib.parse.urlencode(params)
-        params['signature'] = hmac.new(self.secret.encode('utf-8'), query_string.encode('utf-8'),
-                                       hashlib.sha256).hexdigest()
-        url = urljoin(BASE_URL, PATH)
-        print(url)
-        r = requests.get(url, headers=headers, params=params)
-        dataSet = r.json()
-        print(dataSet)
 
     def get_balance(self):
         # Recupérer la balance
@@ -434,7 +515,7 @@ class NaPoleonBinanceSpotBot(object):
         while not position_changed:
             if counter > 1:
                 print(f'position not reached : reupdating positions {positions}')
-            self.cancel_all_orders()
+            self.cancel_all_spot_open_orders()
             position_changed = self.change_position(positions)
             # Prints the current time with a five second difference
             print(f'waiting for the oder to be filled {sleeping_time}')
