@@ -373,7 +373,7 @@ def fetch_local_crypto_hourly_data(ssj=None, local_root_directory=None, hourly_r
     return dataframe
 
 
-def fetch_crypto_hourly_data(ssj=None, local_root_directory=None, hourly_return_pkl_filename_suffix='_hourly_returns.pkl',refetch_all=True, daily_crypto_starting_day='2012-01-01', daily_crypto_ending_day=None, ssj_against='USDT', exchange = 'binance'):
+def fetch_crypto_hourly_data(ssj=None, local_root_directory=None, hourly_return_pkl_filename_suffix='_hourly_returns.pkl',refetch_all=True, daily_crypto_starting_day='2012-01-01', daily_crypto_ending_day=None, ssj_against='USDT', exchange = 'binance', save_to_disk = False):
     dates_stub = daily_crypto_starting_day.strftime('%d_%b_%Y') + '_' + daily_crypto_ending_day.strftime('%d_%b_%Y')
     pickle_saving_path = local_root_directory + ssj + '_' + dates_stub + hourly_return_pkl_filename_suffix
 
@@ -432,6 +432,10 @@ def fetch_crypto_hourly_data(ssj=None, local_root_directory=None, hourly_return_
         dataframe.to_pickle(pickle_saving_path)
     else:
         dataframe = pd.read_pickle(pickle_saving_path)
+
+    if save_to_disk:
+        dataframe.to_pickle(pickle_saving_path)
+
     return dataframe
 
 def fetch_crypto_tuple_hourly_data(ssj=None, pickle_saving_path=None, refetch_all=True, frequency=4):
