@@ -16,6 +16,7 @@ class ParallelRunResultAnalyzer():
         self.db_path_suffix = db_path_suffix
         self.filename =  user + db_path_suffix
         self.db_path = self.local_root_directory + self.filename
+        print(f'reading from dbpath {self.db_path}')
         self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=drop_token,dropbox_backup=dropbox_backup)
 
     def getAllRuns(self):
@@ -149,6 +150,14 @@ class ParallelRunResultAnalyzer():
             if (sqliteConnection):
                 sqliteConnection.close()
                 print("sqlite connection is closed")
+
+        if 'index' in results_df.columns :
+            results_df = results_df.rename(columns={'index':'Date'})
+
+
+        if 'Date' not in results_df.columns :
+            results_df['Date'] = results_df.index
+
         results_df['Date'] = pd.to_datetime(results_df['Date'])
         results_df = results_df.sort_values(by=['Date'])
         results_df = results_df.set_index(results_df['Date'])
