@@ -20,6 +20,9 @@ import numpy as np
 import fastcluster
 from scipy.cluster import hierarchy
 from scipy.cluster.hierarchy import fcluster
+
+from numpy import fft
+
 # TODO : cython
 
 __all__ = ['minDrawdown','ERC', 'HRP', 'IVP', 'MDP', 'MVP', 'MVP_uc', 'rolling_allocation']
@@ -760,6 +763,136 @@ def rolling_forward_looking_allocation(f, X, n=252, s=63, ret=True, drift=True,
 
     return portfolio, w_mat
 
+def fourierExtrapolation(x, n_predict, n_harm=5, detrending=True):
+    n = x.size
+    # number of harmonics in model
+    t = np.arange(0, n)
+    if detrending:
+        p = np.polyfit(t, x, 1)
+        # find linear trend in x
+        pente = p[0]
+    else:
+        pente = 0
+
+    x_notrend = x - pente * t  # detrended x
+    x_freqdom = fft.fft(x_notrend)  # detrended x in frequency domain
+    f = fft.fftfreq(n)  # frequencies
+    indexes = list(range(n))
+    # sort indexes by frequency, lower -> higher
+    indexes.sort(key=lambda i: np.absolute(f[i]))
+
+    t = np.arange(0, n + n_predict)
+    restored_sig = np.zeros(t.size)
+    for i in indexes[:1 + n_harm * 2]:
+        ampli = np.absolute(x_freqdom[i]) / n  # amplitude
+        phase = np.angle(x_freqdom[i])  # phase
+        restored_sig += ampli * np.cos(2 * np.pi * f[i] * t + phase)
+    return restored_sig + pente * t
+
+def fourierFrequency(x):
+    n = x.size
+    # number of harmonics in model
+    x_freqdom = fft.fft(x)  # detrended x in frequency domain
+    f = fft.fftfreq(n)  # frequencies
+    indexes = list(range(n))
+    # sort indexes by frequency, lower -> higher
+    indexes.sort(key=lambda i: np.absolute(f[i]))
+    return f, np.abs(x_freqdom)
+
+def rolling_fourier( close, n=252, s=63, nb_split=3, threshold1=0.15, threshold2=0.15,ls=True ):
+    X = pd.DataFrame(close)#.fillna(method='ffill')
+    idx = X.index
+    portfolio = pd.Series(0., index=idx, name='portfolio')
+    X_ = X
+    roll = rolling._RollingMechanism(idx, n=n, s=s)
+
+    for slice_n, slice_s, slice_s_eval in roll():
+        # Select X
+        sub_X = X_.loc[slice_n].copy()
+        f, module = fourierFrequency(np.asarray(sub_X))
+        specter = {"freq_{:.2f}".format(f[i]): module[i] for i in range(len(f)) if f[i] >= 0}
+        sorted_f = sorted(f)
+        specter_df = pd.DataFrame.from_records(specter)
+        freq_df = specter_df.filter(regex='freq*').copy()
+        freq_columns = list(freq_df.columns)
+        low_freq_columns = freq_columns[:nb_split]
+        high_freq_columns = freq_columns[-nb_split:]
+
+        low_score = freq_df[low_freq_columns].mean(axis=1)[0]
+        high_score = freq_df[high_freq_columns].mean(axis=1)[0]
+        if low_score >= threshold1 or high_score >= threshold2:
+            portfolio.loc[slice_s] = 1.
+        else :
+            if ls:
+                portfolio.loc[slice_s] = -1.
+            else:
+                portfolio.loc[slice_s] = 0.
+
+    return portfolio
+
+def rolling_fourier_extrapolation_simple(close, n=252, s=63, n_harm = 5 , detrending = False):
+    X = pd.DataFrame(close)#.fillna(method='ffill')
+    idx = X.index
+    portfolio = pd.Series(0., index=idx, name='portfolio')
+    X_ = X
+    roll = rolling._RollingMechanism(idx, n=n, s=s)
+    for slice_n, slice_s, slice_s_eval in roll():
+        sub_X = X_.loc[slice_n].copy()
+        prediction = fourierExtrapolation(np.asarray(sub_X).reshape(len(np.asarray(sub_X))),s, n_harm = n_harm, detrending=detrending)
+        final_prediction = prediction[-s:]
+        me_len = slice_s.stop - slice_s.start + 1
+        portfolio.loc[slice_s] = final_prediction[-me_len:]
+    return portfolio
+
+def rolling_fourier_extrapolation( close, n=252, s=63, n_harm = 5 , detrending = False, short = True, quantile_low = 0.05, quantile_high = 0.95):
+    X = pd.DataFrame(close)#.fillna(method='ffill')
+    idx = X.index
+    portfolio = pd.Series(0., index=idx, name='portfolio')
+    portfolio_continuous = pd.Series(0., index=idx, name='portfolio_continuous')
+    X_ = X
+    roll = rolling._RollingMechanism(idx, n=n, s=s)
+    for slice_n, slice_s, slice_s_eval in roll():
+        # Select X
+        #print(slice_n)
+        #print(slice_s)
+        sub_X = X_.loc[slice_n].copy()
+        prediction = fourierExtrapolation(np.asarray(sub_X).reshape(len(np.asarray(sub_X))),s, n_harm = n_harm, detrending=detrending)
+        final_prediction = prediction[-s:]
+        if short:
+            final_prediction_discrete = final_prediction >= 0.
+            pred = 2*final_prediction_discrete.astype(float) - 1
+            me_len = slice_s.stop - slice_s.start+1
+            portfolio.loc[slice_s] = pred[-me_len:]
+            portfolio_continuous.loc[slice_s] = final_prediction[-me_len:]
+        else :
+            final_prediction_discrete = final_prediction >= 0.
+            pred = final_prediction_discrete.astype(float)
+            me_len = slice_s.stop - slice_s.start+1
+            portfolio.loc[slice_s] = pred[-me_len:]
+            portfolio_continuous.loc[slice_s] = final_prediction[-me_len:]
+
+    low_pred = np.quantile(portfolio_continuous, quantile_low)
+    high_pred = np.quantile(portfolio_continuous, quantile_high)
+    if short:
+        def compute_ls_level(low_threshold, high_threshold, row):
+            if row <= low_threshold:
+                return -1.
+            if row >= high_threshold:
+                return 1.
+            return 0.
+        import functools
+        go = functools.partial(compute_ls_level,low_pred,high_pred)
+        portfolio_threshold = portfolio_continuous.apply(go)
+    else:
+        def compute_lo_level(high_threshold, row):
+            if row >= high_threshold:
+                return 1.
+            return 0.
+        import functools
+        go = functools.partial(compute_lo_level, high_pred)
+        portfolio_threshold = portfolio_continuous.apply(go)
+
+    return portfolio_threshold
 
 def rolling_allocation(f, X, n=252, s=63, ret=True, drift=True, filtering_threshold = 0.7, transaction_costs = None, **kwargs):
     r""" Roll an algorithm of portfolio allocation.
